using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

public enum States {Entering, Leaving, Thinking, BeingHungry, Eating};

class Philosopher : Control
{
    private Random r;
    private States state;
    private bool live;
    private Table table;
    private int seat;
    
    public Philosopher(Table t, int seat)
    {
        this.Size = new Size(23, 23);
        this.r = new Random();
        this.table = t;
        this.seat = seat;
        this.live = false;
    }

    public void Start()
    {
        this.live = true;
        ThreadStart ts = new ThreadStart(this.Live);
        Thread t = new Thread(ts);
        this.state = States.Entering;
        t.Start();
    }

    public void Stop()
    {
        this.live = false;
    }
    
    public void Live()
    {
        while(this.live)
        {
            this.Thinking();
            this.state = States.BeingHungry;
            this.Invalidate();
            this.table.DemandForks(this.seat);
            this.Eating();
            this.table.ReleaseForks(this.seat);
        }
        
        this.state = States.Leaving;
        this.Invalidate();
    }
    
    private void Thinking()
    {
        this.state = States.Thinking;
        this.Invalidate();
        Thread.Sleep(this.r.Next(1500, 4000));
    }

    private void Eating()
    {   
        this.state = States.Eating;
        this.Invalidate();
        Thread.Sleep(this.r.Next(1500, 4000));
    }

    protected override void OnPaint(PaintEventArgs e)
    {
        base.OnPaint(e);
        
        switch (this.state)
        {
            case States.Thinking:
                e.Graphics.FillEllipse(Brushes.Green, 1, 1, 20, 20);
                break;
            case States.BeingHungry:
                e.Graphics.FillEllipse(Brushes.Yellow, 1, 1, 20, 20);
                break;
            case States.Eating:
                e.Graphics.FillEllipse(Brushes.Red, 1, 1, 20, 20);
                break;    
            case States.Entering:
            case States.Leaving:
            default:
                e.Graphics.FillEllipse(Brushes.DarkGray, 1, 1, 20, 20);
                break;
        }

        Pen borderPen = new Pen(Brushes.Black, 2.0f);
        e.Graphics.DrawEllipse(borderPen, 1, 1, 20, 20);
        e.Graphics.DrawString(this.seat.ToString(), this.Font, Brushes.Black, new PointF(5.0f, 5.0f));
        borderPen.Dispose();
    }
}
