using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

class Table : Control
{
    private Fork[] forks;
    public Table()
    {
        this.Size = new Size(200, 150);
        
        // init forks, setting locations and directions, then add them to the controls-list
        this.forks = new Fork[5];
        for(int i=0; i<5; i++)
            this.forks[i] = new Fork();
            
        this.forks[0].Location = new Point(this.Width / 2, 5);
        this.forks[1].Location = new Point(this.Width - 25, this.Height / 5 * 1);
        this.forks[2].Location = new Point(this.Width / 5 * 4, this.Height - 35);
        this.forks[3].Location = new Point(this.Width / 5, this.Height - 35);
        this.forks[4].Location = new Point(5, this.Height / 5 * 1);
        
        this.forks[0].direction = Directions.Bottom;
        this.forks[1].direction = Directions.Bottom;
        this.forks[2].direction = Directions.Bottom;
        this.forks[3].direction = Directions.Bottom;
        this.forks[4].direction = Directions.Bottom;
        
        for(int i=0; i<5; i++)
            this.Controls.Add(this.forks[i]);
    }
    
    public void DemandForks(int seat)
    {
        // select forks to reserve
        Fork leftFork = this.forks[seat];
        Fork rightFork;
        if (seat > 0)
            rightFork = this.forks[seat-1];
        else
            rightFork = this.forks[4];

        lock (this)
        {
            while (!(leftFork.isAvailable && rightFork.isAvailable))
                Monitor.Wait(this);

            leftFork.isAvailable = false;
            rightFork.isAvailable = false;
        }
        
        leftFork.Invalidate();
        rightFork.Invalidate();
    }

    public void ReleaseForks(int seat)
    {
        // select forks to reserve
        Fork leftFork = this.forks[seat];
        Fork rightFork;
        if (seat > 0)
            rightFork = this.forks[seat - 1];
        else
            rightFork = this.forks[4];

        lock (this)
        {
            while (leftFork.isAvailable && rightFork.isAvailable)
                Monitor.Wait(this);

            leftFork.isAvailable = true;
            rightFork.isAvailable = true;
            
            Monitor.PulseAll(this);
        }

        leftFork.Invalidate();
        rightFork.Invalidate();
    }

    protected override void OnPaint(PaintEventArgs e)
    {
        base.OnPaint(e);
        
        Pen borderPen = new Pen(Brushes.Black, 2.0f);
        
        // draw the table itself
        e.Graphics.DrawRectangle(borderPen, 1, 1, this.Width - 2, this.Height - 2);
        
        // draw the bowl in the center
        Point[] p = new Point[6];
        int w = this.Width / 5;
        p[0] = new Point(this.Width / 2 - w, this.Height / 2 - 20);
        p[1] = new Point(this.Width / 2 - w + 10, this.Height / 2);
        p[2] = new Point(this.Width / 2 + w - 10, this.Height / 2);
        p[3] = new Point(this.Width / 2 + w, this.Height / 2 - 20);
        p[4] = new Point(this.Width / 2 + w, this.Height / 2 - 15);
        p[5] = new Point(this.Width / 2 - w, this.Height / 2 - 15);
        e.Graphics.DrawPolygon(borderPen, p);
        
        borderPen.Dispose();
    }
}